@@ -67,6 +67,9 @@
 #include "ImageIOWizardModel.h"
 #include "IntensityCurveInterface.h"
 #include "ColorLabelQuickListModel.h"
+#include "InterpolateLabelModel.h"
+#include "RegistrationModel.h"
+#include "InteractiveRegistrationModel.h"
 
 #include <itksys/SystemTools.hxx>
 
@@ -97,6 +100,10 @@ GlobalUIModel::GlobalUIModel()
   m_PaintbrushSettingsModel = PaintbrushSettingsModel::New();
   m_PaintbrushSettingsModel->SetParentModel(this);
 
+  // Registration model
+  m_RegistrationModel = RegistrationModel::New();
+  m_RegistrationModel->SetParentModel(this);
+
   // Create the slice models
   for (unsigned int i = 0; i < 3; i++)
     {
@@ -117,6 +124,10 @@ GlobalUIModel::GlobalUIModel()
 
     m_AnnotationModel[i] = AnnotationModel::New();
     m_AnnotationModel[i]->SetParent(m_SliceModel[i]);
+
+    m_InteractiveRegistrationModel[i] = InteractiveRegistrationModel::New();
+    m_InteractiveRegistrationModel[i]->SetParent(m_SliceModel[i]);
+    m_InteractiveRegistrationModel[i]->SetRegistrationModel(m_RegistrationModel);
     }
 
 
@@ -195,6 +206,10 @@ GlobalUIModel::GlobalUIModel()
   m_ColorLabelQuickListModel = ColorLabelQuickListModel::New();
   m_ColorLabelQuickListModel->SetParentModel(this);
 
+  // Interpolation dialog
+  m_InterpolateLabelModel = InterpolateLabelModel::New();
+  m_InterpolateLabelModel->SetParentModel(this);
+
   // Set up the cursor position model
   m_CursorPositionModel = wrapGetterSetterPairAsProperty(
         this,
@@ -492,9 +507,10 @@ void GlobalUIModel::LoadUserPreferences()
   dbs->ReadFromRegistry(
         si->Folder("UserInterface.DefaultBehavior"));
 
-  // Read the global display properties
-  m_GlobalDisplaySettings->ReadFromRegistry(
-        si->Folder("SliceView.DisplaySettings"));
+  // Read the global display properties and apply them
+  SmartPtr<GlobalDisplaySettings> temp_gds = GlobalDisplaySettings::New();
+  temp_gds->ReadFromRegistry(si->Folder("SliceView.DisplaySettings"));
+  this->SetGlobalDisplaySettings(temp_gds);
 
   // Read the 3D mesh options
   m_Driver->GetGlobalState()->GetMeshOptions()->ReadFromRegistry(
