@@ -424,7 +424,7 @@ void SnakeWizardModel::SetForegroundClassColorLabelValue(LabelType value)
 bool SnakeWizardModel::GetForestSizeValueAndRange(int &value, NumericValueRange<int> *range)
 {
   // Must have a classification engine
-  RFClassificationEngine *rfe = m_Driver->GetClassificationEngine();
+  IRISApplication::RFEngine *rfe = m_Driver->GetClassificationEngine();
   if(!rfe)
     return false;
 
@@ -437,7 +437,7 @@ bool SnakeWizardModel::GetForestSizeValueAndRange(int &value, NumericValueRange<
 
 void SnakeWizardModel::SetForestSizeValue(int value)
 {
-  RFClassificationEngine *rfe = m_Driver->GetClassificationEngine();
+  IRISApplication::RFEngine *rfe = m_Driver->GetClassificationEngine();
   assert(rfe);
 
   rfe->SetForestSize(value);
@@ -448,7 +448,7 @@ void SnakeWizardModel::SetForestSizeValue(int value)
 bool SnakeWizardModel::GetTreeDepthValueAndRange(int &value, NumericValueRange<int> *range)
 {
   // Must have a classification engine
-  RFClassificationEngine *rfe = m_Driver->GetClassificationEngine();
+  IRISApplication::RFEngine *rfe = m_Driver->GetClassificationEngine();
   if(!rfe)
     return false;
 
@@ -461,7 +461,7 @@ bool SnakeWizardModel::GetTreeDepthValueAndRange(int &value, NumericValueRange<i
 
 void SnakeWizardModel::SetTreeDepthValue(int value)
 {
-  RFClassificationEngine *rfe = m_Driver->GetClassificationEngine();
+  IRISApplication::RFEngine *rfe = m_Driver->GetClassificationEngine();
   assert(rfe);
 
   rfe->SetTreeDepth(value);
@@ -472,12 +472,12 @@ void SnakeWizardModel::SetTreeDepthValue(int value)
 bool SnakeWizardModel::GetClassifierPatchRadiusValueAndRange(int &value, NumericValueRange<int> *range)
 {
   // Must have a classification engine
-  RFClassificationEngine *rfe = m_Driver->GetClassificationEngine();
+  IRISApplication::RFEngine *rfe = m_Driver->GetClassificationEngine();
   if(!rfe)
     return false;
 
   // Get the value
-  RFClassificationEngine::RadiusType radius = rfe->GetPatchRadius();
+  IRISApplication::RFEngine::RadiusType radius = rfe->GetPatchRadius();
   value = (int) radius[0];
 
   if(range)
@@ -488,11 +488,11 @@ bool SnakeWizardModel::GetClassifierPatchRadiusValueAndRange(int &value, Numeric
 
 void SnakeWizardModel::SetClassifierPatchRadiusValue(int value)
 {
-  RFClassificationEngine *rfe = m_Driver->GetClassificationEngine();
+  IRISApplication::RFEngine *rfe = m_Driver->GetClassificationEngine();
   assert(rfe);
   assert(value >= 0);
 
-  RFClassificationEngine::RadiusType radius;
+  IRISApplication::RFEngine::RadiusType radius;
   radius.Fill((unsigned int) value);
   rfe->SetPatchRadius(radius);
 
@@ -502,12 +502,12 @@ void SnakeWizardModel::SetClassifierPatchRadiusValue(int value)
 bool SnakeWizardModel::GetClassifierBiasValueAndRange(double &value, NumericValueRange<double> *range)
 {
   // Must have a classification engine
-  RFClassificationEngine *rfe = m_Driver->GetClassificationEngine();
+  IRISApplication::RFEngine *rfe = m_Driver->GetClassificationEngine();
   if(!rfe)
     return false;
 
   // Must have a classifier
-  RandomForestClassifier *rfc = rfe->GetClassifier();
+  IRISApplication::RFClassifier *rfc = rfe->GetClassifier();
   if(!rfc)
     return false;
 
@@ -524,10 +524,10 @@ bool SnakeWizardModel::GetClassifierBiasValueAndRange(double &value, NumericValu
 
 void SnakeWizardModel::SetClassifierBiasValue(double value)
 {
-  RFClassificationEngine *rfe = m_Driver->GetClassificationEngine();
+  IRISApplication::RFEngine *rfe = m_Driver->GetClassificationEngine();
   assert(rfe);
 
-  RandomForestClassifier *rfc = rfe->GetClassifier();
+  IRISApplication::RFClassifier *rfc = rfe->GetClassifier();
   assert(rfc);
 
   rfc->SetBiasParameter(value);
@@ -541,7 +541,7 @@ void SnakeWizardModel::SetClassifierBiasValue(double value)
 bool SnakeWizardModel::GetClassifierUseCoordinatesValue(bool &value)
 {
   // Must have a classification engine
-  RFClassificationEngine *rfe = m_Driver->GetClassificationEngine();
+  IRISApplication::RFEngine *rfe = m_Driver->GetClassificationEngine();
   if(!rfe)
     return false;
 
@@ -551,7 +551,7 @@ bool SnakeWizardModel::GetClassifierUseCoordinatesValue(bool &value)
 
 void SnakeWizardModel::SetClassifierUseCoordinatesValue(bool value)
 {
-  RFClassificationEngine *rfe = m_Driver->GetClassificationEngine();
+  IRISApplication::RFEngine *rfe = m_Driver->GetClassificationEngine();
   assert(rfe);
   rfe->SetUseCoordinateFeatures(value);
   InvokeEvent(RFClassifierModifiedEvent());
@@ -562,7 +562,7 @@ bool SnakeWizardModel
     ClassifierLabelForegroundMap &value, ClassifierLabelForegroundMapDomain *range)
 {
   // Get the classification engine
-  RFClassificationEngine *rfengine = m_Driver->GetClassificationEngine();
+  IRISApplication::RFEngine *rfengine = m_Driver->GetClassificationEngine();
   if(!rfengine || !rfengine->GetClassifier()->IsValidClassifier())
     return false;
 
@@ -572,11 +572,11 @@ bool SnakeWizardModel
     range->clear();
 
   // Add all of the labels
-  RandomForestClassifier *rfc = rfengine->GetClassifier();
-  const RandomForestClassifier::WeightArray &wa = rfc->GetClassWeights();
+  IRISApplication::RFClassifier *rfc = rfengine->GetClassifier();
+  const IRISApplication::RFClassifier::WeightArray &wa = rfc->GetClassWeights();
   for(size_t i = 0; i < wa.size(); i++)
     {
-    RandomForestClassifier::MappingType::const_iterator itl = rfc->GetClassToLabelMapping().find(i);
+    IRISApplication::RFClassifier::MappingType::const_iterator itl = rfc->GetClassToLabelMapping().find(i);
     if(itl != rfc->GetClassToLabelMapping().end())
       {
       LabelType label = itl->second;
@@ -594,15 +594,15 @@ void SnakeWizardModel
     ClassifierLabelForegroundMap value)
 {
   // Get the classification engine
-  RFClassificationEngine *rfengine = m_Driver->GetClassificationEngine();
+  IRISApplication::RFEngine *rfengine = m_Driver->GetClassificationEngine();
   assert(rfengine && rfengine->GetClassifier()->IsValidClassifier());
 
   // Check if anything was actually modified - to avoid loops
   bool changed = false;
 
   // Update the weight of each class
-  RandomForestClassifier *rfc = rfengine->GetClassifier();
-  for(RandomForestClassifier::MappingType::const_iterator it = rfc->GetClassToLabelMapping().begin();
+  IRISApplication::RFClassifier *rfc = rfengine->GetClassifier();
+  for(IRISApplication::RFClassifier::MappingType::const_iterator it = rfc->GetClassToLabelMapping().begin();
       it != rfc->GetClassToLabelMapping().end(); ++it)
     {
     double old_weight = rfc->GetClassWeights()[it->first];
@@ -672,11 +672,13 @@ bool SnakeWizardModel::CanGenerateSpeedVolume()
       return true;
     case PREPROCESS_RF:
       {
-      RFClassificationEngine *cfe = m_Driver->GetClassificationEngine();
-      RandomForestClassifier *rfc = cfe->GetClassifier();
+      IRISApplication::RFEngine *cfe = m_Driver->GetClassificationEngine();
+      IRISApplication::RFClassifier *rfc = cfe->GetClassifier();
       return rfc->IsValidClassifier();
       }
     }
+
+  return false;
 }
 
 ScalarImageWrapperBase *SnakeWizardModel::GetSelectedScalarLayer()
@@ -1166,10 +1168,26 @@ void SnakeWizardModel::SetPreprocessingModeValue(PreprocessingMode value)
 void SnakeWizardModel::CompletePreprocessing()
 {
   // If we are in classification pre-segmentation mode, set the active drawing label
-  // to match the foreground class - otherwise it's confusing to the user
+  // to match the foreground class(es) - otherwise it's confusing to the user
   if(m_Driver->GetPreprocessingMode() == PREPROCESS_RF)
-    m_Parent->GetGlobalState()->SetDrawingColorLabel(
-          this->GetClassiferFirstForegroundLabel());
+    {
+    // What is the current drawing label?
+    LabelType label = m_Parent->GetGlobalState()->GetDrawingColorLabel();
+
+    // Is the current label one of the foreground classes?
+    ClassifierLabelForegroundMap fbtable = this->GetClassifierLabelForeground();
+    ClassifierLabelForegroundMap::const_iterator itLabel = fbtable.find(label);
+    if(itLabel == fbtable.end() || itLabel->second == false)
+      {
+      // No, it's not one of the foreground classes. So find the first foreground class
+      for(ClassifierLabelForegroundMap::const_iterator it = fbtable.begin(); it != fbtable.end(); ++it)
+        if(it->second)
+          {
+          m_Parent->GetGlobalState()->SetDrawingColorLabel(it->first);
+          break;
+          }
+      }
+    }
 
   // Disconnect preview pipeline
   m_Driver->EnterPreprocessingMode(PREPROCESS_NONE);
@@ -1449,9 +1467,6 @@ void SnakeWizardModel::OnEvolutionPageFinish()
   // Update IRIS with SNAP images
   m_Driver->UpdateIRISWithSnapImageData(NULL);
 
-  // Set an undo point in IRIS, not SNAP!
-  m_Driver->GetIRISImageData()->StoreUndoPoint("Automatic Segmentation");
-
   // Return to IRIS mode
   m_Driver->SetCurrentImageDataToIRIS();
   m_Driver->ReleaseSNAPImageData();
@@ -1608,7 +1623,7 @@ void SnakeWizardModel::TagRFPreprocessingFilterModified()
   // TODO: this is not the right way to do this! Make RandomForestClassifier an itkObject
   // and an inout to the filter, so we don't have to update the filter itself!!
   // THIS IS HACKY!!!
-  RFClassificationEngine *ce = m_Driver->GetClassificationEngine();
+  IRISApplication::RFEngine *ce = m_Driver->GetClassificationEngine();
   typedef SlicePreviewFilterWrapper<RFPreprocessingFilterConfigTraits>
                                             RFPreprocessingPreviewWrapperType;
   RFPreprocessingPreviewWrapperType *junk =
@@ -1782,13 +1797,13 @@ LabelType SnakeWizardModel::GetClassiferFirstForegroundLabel()
 void SnakeWizardModel::TrainClassifier()
 {
   // Get the classification engine
-  RFClassificationEngine *rfengine = m_Driver->GetClassificationEngine();
+  IRISApplication::RFEngine *rfengine = m_Driver->GetClassificationEngine();
 
   // Perform the classification
   rfengine->TrainClassifier();
 
   // Create a list of utilized labels
-  RandomForestClassifier *rfc = rfengine->GetClassifier();
+  IRISApplication::RFClassifier *rfc = rfengine->GetClassifier();
 
   // Fire the appropriate event
   InvokeEvent(RFClassifierModifiedEvent());
@@ -1800,7 +1815,7 @@ void SnakeWizardModel::TrainClassifier()
 bool SnakeWizardModel::IsClassifierTrained()
 {
   // Get the classification engine
-  RFClassificationEngine *rfengine = m_Driver->GetClassificationEngine();
+  IRISApplication::RFEngine *rfengine = m_Driver->GetClassificationEngine();
   return rfengine && rfengine->GetClassifier()->IsValidClassifier();
 }
 
