@@ -15,6 +15,7 @@
 #include "SliceWindowCoordinator.h"
 #include "PolygonDrawingModel.h"
 #include "AnnotationModel.h"
+#include "InteractiveRegistrationModel.h"
 #include "AnnotationRenderer.h"
 #include "QtWidgetActivator.h"
 #include "SnakeModeRenderer.h"
@@ -24,6 +25,7 @@
 #include "SliceWindowDecorationRenderer.h"
 #include "LayerInspectorDialog.h"
 #include "MainImageWindow.h"
+#include "RegistrationRenderer.h"
 #include "SNAPQtCommon.h"
 #include "QtScrollbarCoupling.h"
 #include "QtSliderCoupling.h"
@@ -188,6 +190,7 @@ void SliceViewPanel::Initialize(GlobalUIModel *model, unsigned int index)
   ui->imSnakeROI->SetModel(m_GlobalUI->GetSnakeROIModel(index));
   ui->imPaintbrush->SetModel(m_GlobalUI->GetPaintbrushModel(index));
   ui->imAnnotation->SetModel(m_GlobalUI->GetAnnotationModel(index));
+  ui->imRegistration->SetModel(m_GlobalUI->GetInteractiveRegistrationModel(index));
 
   // ui->labelQuickList->SetModel(m_GlobalUI);
 
@@ -224,6 +227,9 @@ void SliceViewPanel::Initialize(GlobalUIModel *model, unsigned int index)
   // Listen to annotation changes
   connectITK(m_GlobalUI->GetAnnotationModel(index), ModelUpdateEvent());
 
+  // Listen to registration changes
+  connectITK(m_GlobalUI->GetInteractiveRegistrationModel(index), ModelUpdateEvent());
+
   // Listen to all (?) events from the snake wizard as well
   connectITK(m_GlobalUI->GetSnakeWizardModel(), IRISEvent());
 
@@ -400,6 +406,10 @@ void SliceViewPanel::OnToolbarModeChange()
       ConfigureEventChain(ui->imAnnotation);
       ovTiled.push_back(ui->imAnnotation->GetRenderer());
       break;
+    case REGISTRATION_MODE:
+      ConfigureEventChain(ui->imRegistration);
+      ovTiled.push_back(ui->imRegistration->GetRenderer());
+      break;
     case ROI_MODE:
       ConfigureEventChain(ui->imSnakeROI);
       ovTiled.push_back(ui->imSnakeROI->GetRenderer());
