@@ -111,7 +111,8 @@ enum ToolbarModeType
   POLYGON_DRAWING_MODE,
   PAINTBRUSH_MODE,
   ROI_MODE,
-  ANNOTATION_MODE
+  ANNOTATION_MODE,
+  REGISTRATION_MODE
 };
 
 enum ToolbarMode3DType
