@@ -72,6 +72,9 @@ public:
   typedef ImageWrapperBase::DisplaySliceType DisplaySliceType;
   typedef ImageWrapperBase::DisplaySlicePointer DisplaySlicePointer;
 
+  typedef InputSliceType::PixelType InputPixelType;
+  typedef DisplaySliceType::PixelType DisplayPixelType;
+
   /**
    * Set the table of color labels used to produce color slice images
    */
@@ -93,6 +96,8 @@ public:
   virtual void Save(Registry &folder) {}
   virtual void Restore(Registry &folder) {}
 
+  virtual DisplayPixelType MapPixel(const InputPixelType &val);
+
 protected:
 
   ColorLabelTableDisplayMappingPolicy();
@@ -249,6 +254,8 @@ public:
   virtual void Save(Registry &folder);
   virtual void Restore(Registry &folder);
 
+  virtual DisplayPixelType MapPixel(const PixelType &val);
+
 
 protected:
 
@@ -325,6 +332,11 @@ struct MultiChannelDisplayMode
    */
   bool UseRGB;
 
+  /**
+   * Special mode for rendering 3-component images as displacement grids
+   */
+  bool RenderAsGrid;
+
   /**
    * When not in RGB mode, which scalar representation is selected for
    * display. Only used if UseRGB is false.
@@ -341,7 +353,7 @@ struct MultiChannelDisplayMode
   MultiChannelDisplayMode();
 
   /** Default constructor - select first component */
-  MultiChannelDisplayMode(bool use_rgb,
+  MultiChannelDisplayMode(bool use_rgb, bool render_as_grid,
                           ScalarRepresentation rep,
                           int comp = 0);
 
@@ -428,7 +440,9 @@ public:
 
   irisGetMacro(ColorMap, ColorMap *)
 
-  protected:
+  DisplayPixelType MapPixel(const PixelType &xin);
+
+protected:
 
   LinearColorMapDisplayMappingPolicy();
   virtual ~LinearColorMapDisplayMappingPolicy();
@@ -526,6 +540,8 @@ public:
 
   irisGetMacro(ScalarRepresentation, ScalarImageWrapperBase *)
 
+  DisplayPixelType MapPixel(const PixelType &val);
+
 protected:
 
   MultiChannelDisplayMappingPolicy();
